<?php

namespace App\Service;

class Order {

    private $util;
    private $orderFactory;

    public function __construct(\App\Util\Util $util, 
                                \App\Factory\Order $orderFactory) {
        $this->util = $util;
        $this->orderFactory = $orderFactory;
    }

    public function history($params) {
        $uppercaseKeysArray = [];
        foreach ($params as $key => $value) {
            $uppercaseKey = strtoupper($key);
            $uppercaseKeysArray[$uppercaseKey] = $value;
        }

        $paramNames = ['DTINICIAL','DTFINAL', 'DTINIAGENDADA', 'DTFIMAGENDADA', 'CDFILIAL',
            'NRVENDAREST', 'NRCOMANDA', 'CLIENTECONSUMIDOR', 'IDSINCMSDE', 'IDRETBALLOJA',
            'IDORGCMDVENDA', 'ORDERSTYPE', 'SCHEDULEDTAB', 'IDSTCOMANDA', 'IDSINCAGENDA'];

        $cleanedParams = [];

        foreach ($paramNames as $paramName) {
            if (array_key_exists($paramName, $uppercaseKeysArray) && !empty($uppercaseKeysArray[$paramName])) {
                if (is_array($uppercaseKeysArray[$paramName])) {
                    $cleanedParams[$paramName] = implode(',', $uppercaseKeysArray[$paramName]);
                } else {
                    $cleanedParams[$paramName] = $uppercaseKeysArray[$paramName];
                }
            } else {
                $cleanedParams[$paramName] = 'T';
            }
        }
        $isScheduled = false;

        if ($cleanedParams['IDSINCAGENDA'] === 'N') {
            $cleanedParams['IDSTCOMANDA'] = 'A';
            $isScheduled = true;
        }

        return $this->orderFactory->history($cleanedParams, $isScheduled);
    }

    public function errorLogs($paramsLog) {

        $errorLogs = $this->orderFactory->errorLogs($paramsLog);
       
        return $errorLogs;
    }

    public function getOrder($params)
    {
        if (isset($params['order'])) {
			$orderByColumns = array();
			foreach ($params['order'] as $column) {
					$column = str_replace('.', ' ', $column);
					$orderByColumns[] = strtoupper($column);
			}
		return $sortOptions = "
			 ORDER BY " . implode(", ", $orderByColumns);
		}else{
        return $sortOptions = " ORDER BY DTINCLUSAO ASC ";
        }
			
    }

    public function getParamsValue($data, $indexName) {
        $values = [];
    
        if (isset($data[$indexName])) {
            foreach ($data[$indexName] as $item) {
                if (isset($item[$indexName])) {
                    $values[] = $item[$indexName];
                }
            }
            return implode(',', $values);
        }
    
        return null;
    }

    public function ensureDTInclusaoExists(&$params) {
        if (!isset($params["DTINCLUSAO"])) {
            $params["DTINCLUSAO"] = [null, null];
        }
        return $params["DTINCLUSAO"];
    }

    public function checkOldLogs()
    {
        try {
            $this->orderFactory->deleteOldLogs($this->util->getEnv("PROJECT_DELETE_LOG_MONTH"));
            $this->orderFactory->updateOldLogs($this->util->getEnv("PROJECT_UPDATE_LOG_MONTH"));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cancel($cdFilial, $cdOperador, $nrComanda, $nrVendaRest, $dsCancMotiv)
    {
        $cdVendedor = $this->orderFactory->getSeller($cdOperador);
        if (!$cdVendedor) {
            throw new \Exception("Operador não autorizado para cancelar pedidos.");
        }
        $orderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrVendaRest, $nrComanda);

        if (empty($orderStatus[0]['IDSTCOMANDA'])) {
            throw new \Exception("Pedido não encontrado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == "4") {
            throw new \Exception("Pedido já cancelado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == 'P' || $orderStatus[0]['IDSTCOMANDA'] == '5' || $orderStatus[0]['IDSTCOMANDA'] == '6') {
            throw new \Exception("Este pedido já foi impresso. Para cancelá-lo, utilize o Gestão de Vendas.");
        }

        $dsCancMotiv = $this->util->removeAcentos($dsCancMotiv);
        $dsCancMotiv = preg_replace('/[^A-Za-z0-9. -]/', '', $dsCancMotiv);
        $dsCancMotiv = strtoupper($dsCancMotiv);

        $cancelOrder = $this->orderFactory->cancelOrder($cdFilial, $nrVendaRest, $nrComanda, $cdOperador, $dsCancMotiv);
        $forceOrderSync = $this->orderFactory->forceOrderSync($cdFilial, $nrVendaRest);

        if ($cancelOrder->rowCount() === 0 || $forceOrderSync->rowCount() === 0) {
            throw new \Exception("Ocorreu um erro ao cancelar o pedido.");
        }

        return "Pedido cancelado com sucesso.";        
    }

    public function getBranchList()
    {

        $branchList = $this->orderFactory->getBranchList();

        return $branchList;
    }

    public function getOrderNumber($search)
    {

        $orderNumber = $this->orderFactory->getOrderNumber($search);

        return $orderNumber;
    }

    public function deleteLogs($CDINTEGRACAOERR)
    {
        $this->orderFactory->deleteLogs($CDINTEGRACAOERR);
    }

    public function getConsumers($params) {
        $searchTerm = isset($params['consumers']) ? $params['consumers'] : '';       
        return $this->orderFactory->getConsumers($searchTerm);        
    }

    public function selectTableInteg()
    {

        $selectTableInteg = $this->orderFactory->selectTableInteg();

        return $selectTableInteg;
    }

    public function selectTableFilial()
    {
        $selectTableFilial = $this->orderFactory->selectTableFilial();

        return $selectTableFilial;
    }

    public function getLogDetails($cdIntegracaoErr)
    {

        $getLogDetails = $this->orderFactory->getLogDetails($cdIntegracaoErr);

        if ($getLogDetails) {
            $dsArqPed = $getLogDetails[0]['DSARQPED'];

            if (empty($dsArqPed)) {
                return '';
            }

            $username = $this->util->getEnv("USERNAME_LOG");
            $password = $this->util->getEnv("PASSWORD_LOG");
            $fixedPartUrl = $this->util->getEnv("FIXED_LOG_URL");

            $completeUrl = $fixedPartUrl . $dsArqPed;

            $authHeader = "Authorization: Basic " . base64_encode("$username:$password");

            $ch = curl_init($completeUrl);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array($authHeader));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            $fileContent = curl_exec($ch);

            if (curl_errno($ch)) {
                throw new \Exception("Erro na solicitação cURL: " . curl_error($ch));
            }   

            curl_close($ch);

            $json_data = json_decode($fileContent);

            if ($json_data !== null) {
                return $fileContent;
            } else {
                return '';
            }
        } else {
            throw new \Exception("Nenhum registro encontrado.");
        }
    }


    public function getOrderDetails($params)
    {
        $valueElementOrderDetails = $this->orderFactory->getOrderDetails($params);

        return $valueElementOrderDetails;
    }

    public function getOrderAddress($params)
    {
        $orderAddress = $this->orderFactory->getOrderAddress($params);

        return $orderAddress;
    }

    public function getPaymentMethods($params)
    {
        $paymentMethods = $this->orderFactory->getPaymentMethods($params);
        foreach ($paymentMethods as &$orderPayment) {
            if ($orderPayment['IDTIPOMOVIVEDLV'] == 'S') {
                $orderPayment['NMTIPORECE'] = 'TROCO';
            }
        }
        return $paymentMethods;
    }

    public function getOrderItemsRepo($params)
    {
        $products = array();
        $combos = $this->orderFactory->getOrderCombos($params);
        $items  = $this->orderFactory->getOrderItemsRepo($params);

        foreach ($combos as &$combo) {
            $combo['ISCOMBO'] = true;
            array_push($products, $combo);
        }

        foreach ($items as &$item) {
            $item['ISCOMBO'] = false;
            $item['QTPRODCOMVEN'] = floatval($item['QTPRODCOMVEN']);
            array_push($products, $item);
        }

        foreach ($products as &$product) {
            $product['TOTAL']  = (($product['VRPRECCOMVEN'] * $product['QTPRODCOMVEN']) + $product['VRACRCOMVEN'] - $product['VRDESCCOMVEN']);
        }

        return $products;
    }

    public function getSubItens($params)
    {
        $subItens = $this->orderFactory->getSubItens($params);
        foreach ($subItens as &$item) {
            $item['TOTAL'] = (($item['VRPRECCOMVEN'] * $item['QTPRODCOMVEN']) - $item['VRDESCCOMVEN'] + $item['VRACRCOMVEN']);
        }
        return $subItens;
    }

    public function getItemObservation($params)
    {
        $explodedObs       = array();
        $observations      = $this->orderFactory->getItemObservation($params);
        $otherObservation  = $this->orderFactory->getOtherItemObservation($params);

        if ($otherObservation) {
            $explodedObs = explode(";", $otherObservation[0]['DSOCORR']);
            foreach ($explodedObs as $obsString) {
                $obsString = trim($obsString);
                $isRegularObservation = false;

                if (!empty($obsString)) {
                    foreach ($observations as $observation) {
                        if ($observation['DSOCORR'] == $obsString) {
                            $isRegularObservation = true;
                        }
                    }
                    if (!$isRegularObservation) {
                        $otherObservation['DSOCORR'] = $obsString;
                        array_push($observations, $otherObservation);
                    }
                }
            }
        }
        return $observations;
    }

    public function dashboard($dtInicAberMesa, $dtFimAberMesa)
    {
        try {
            $dashboard = $this->orderFactory->dashboard($dtInicAberMesa, $dtFimAberMesa);

            return $dashboard;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function forwardOrder($cdFilial, $nrComanda, $nrVendaRest)
    {
        $orderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrVendaRest, $nrComanda);

        if (empty($orderStatus[0]['IDSTCOMANDA'])) {
            throw new \Exception("Pedido não encontrado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == "4") {
            throw new \Exception("Pedido já cancelado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == 'P' || $orderStatus[0]['IDSTCOMANDA'] == '5' || $orderStatus[0]['IDSTCOMANDA'] == '6') {
            throw new \Exception("Este pedido já foi impresso.");
        }

        $this->orderFactory->forwardOrderComandaven($cdFilial, $nrComanda, $nrVendaRest);
        $this->orderFactory->forwardOrderVendarest($cdFilial, $nrVendaRest);
        $this->orderFactory->forceOrderSync($cdFilial, $nrVendaRest);
    }
}
